Package: GARCH.X
Title: Estimation and Exogenous Covariate Selection for ARCH-m(X),
        Additive ARCH-m(x), and GARCH-X Models
Version: 2.0
Description: Estimates the parameters and nonparametric functions of an ARCH-m(X) model with exogenous covariates, estimates the parameters and nonparametric functions of an Additive ARCH-m(X) model with exogenous covariates, estimates the parameters of a GARCH-X model with exogenous covariates, performs hypothesis tests for the covariates returning the p-values, and performs stepwise variable selection on the exogenous covariates, and uses False Discovery Rate p-value corrections to select the exogenous variables.
Authors@R: c(person(given = "Adriano",
                        family = "Zambom",
                        role = c("aut", "cre"),
                        email = "adriano.zambom@csun.edu"),
                 person(given = "Vincent",
                        family = "Alegrete",
                        role = "aut"),
                 person(given = "Elijah",
                        family = "Sagaran",
                        role = "aut"),
                 person(given = "Avni",
                        family = "Israni",
                        role = "aut"))
License: GPL (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: stats, methods, nnls, utils, GA, GenSA, pso, KernSmooth
NeedsCompilation: no
Packaged: 2026-04-21 16:38:05 UTC; azambom
Author: Adriano Zambom [aut, cre],
  Vincent Alegrete [aut],
  Elijah Sagaran [aut],
  Avni Israni [aut]
Maintainer: Adriano Zambom <adriano.zambom@csun.edu>
Repository: CRAN
Date/Publication: 2026-04-21 17:12:08 UTC
Built: R 4.6.0; ; 2026-04-27 09:33:17 UTC; unix
