# Generated by roxygen2: do not edit by hand

S3method(predict, ARCHmX)
S3method(predict, GARCHX)
S3method(predict, ARCHmXAdditive)
S3method(summary, ARCHmX)
S3method(summary, GARCHX)
S3method(summary, ARCHmXAdditive)
S3method(AIC, ARCHmX)
S3method(AIC, GARCHX)
S3method(AIC, ARCHmXAdditive)
S3method(BIC, ARCHmX)
S3method(BIC, GARCHX)
S3method(BIC, ARCHmXAdditive)
export(sim.ARCHmX)
export(sim.GARCHX)
export(sim.ARCHmXAdditive)
export(ARCHmX)
export(GARCHX)
export(ARCHmXAdditive)
export(stepwise)
importFrom("methods", "is")
importFrom("GA", "ga")
importFrom("GenSA", "GenSA")
importFrom("pso", "psoptim")
importFrom("nnls", "nnls")
importFrom("stats", "AIC", "BIC", "coef", "lm", "nlminb", "optim", "p.adjust", "pchisq", "pnorm", "predict", "rnorm", "rt", "runif", "sd", "var", "approx", "approxfun", "lowess")
importFrom("utils", "combn")
importFrom("KernSmooth", "locpoly")