GARCHX_check_order <- function(n, t, arr){
  # Used in GARCHX_variance function to get
  # h.entry or eps.entry

  # Check if n == 0
  if(n == 0){
    # If so, val is null
    val <- NULL
  } else {
    # If not, assign arr values to val
    val <- arr[(t-1):(t-n)]
  } # End if-else

  # Return val
  return(val)
} # End Check_p
