GARCHX <- function(eps, X, order = c(1, 1), delta = 2, covariates = "all", optim.method = "NR"){
  validate_GARCHX(eps, X, order)
  obj <- new_GARCHX(eps, X, order, delta, covariates, optim.method)
  return(obj)
}