GARCHX_check_gamma <- function(d, t, X){
  # Used in GARCHX_variance to find X.entry

  # Checks if d is 0
  if(d == 0){
    # If yes, val is null
    val <- NULL
  } else {
    # If not, assign the row of exogenous covariates to val
    val <- X[(t-1),]
  } # End if-else

  # Return val
  return(val)
} # End check_pi
